from __future__ import unicode_literals

from django.apps import AppConfig
from django.db.models.functions import Now
from django.utils.translation import ugettext_lazy as _


class SchedulerConfig(AppConfig):
    name = 'scheduler'
    verbose_name = _('Django RQ Scheduler')

    def ready(self):
        try:
            self.reschedule_cron_jobs()
            self.reschedule_repeatable_jobs()
            self.reschedule_scheduled_jobs()
        except:
            # Django isn't ready yet, example a management command is being
            # executed
            pass

    def reschedule_cron_jobs(self):
        CronJob = self.get_model('CronJob')
        jobs = CronJob.objects.filter(enabled=True)
        self.reschedule_jobs(jobs)

    def reschedule_repeatable_jobs(self):
        RepeatableJob = self.get_model('RepeatableJob')
        jobs = RepeatableJob.objects.filter(enabled=True)
        self.reschedule_jobs(jobs)

    def reschedule_scheduled_jobs(self):
        ScheduledJob = self.get_model('ScheduledJob')
        jobs = ScheduledJob.objects.filter(
            enabled=True, scheduled_time__lte=Now())
        self.reschedule_jobs(jobs)

    def reschedule_jobs(self, jobs):
        for job in jobs:
            if job.is_scheduled() is False:
                job.save()



# -*- coding: utf-8 -*-
# Generated by Django 1.9.2 on 2016-03-02 15:28
from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='RepeatableJob',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('name', models.CharField(max_length=128, unique=True, verbose_name='name')),
                ('callable', models.CharField(max_length=2048, verbose_name='callable')),
                ('enabled', models.BooleanField(default=True, verbose_name='enabled')),
                ('queue', models.CharField(choices=[(b'default', b'default'), (b'high', b'high'), (b'low', b'low')], max_length=16, verbose_name='queue')),
                ('job_id', models.CharField(blank=True, editable=False, max_length=128, null=True, verbose_name='job id')),
                ('scheduled_time', models.DateTimeField(verbose_name='scheduled time')),
                ('interval', models.PositiveIntegerField(verbose_name='interval')),
                ('interval_unit', models.CharField(choices=[('minutes', 'minutes'), ('hours', 'hours'), ('days', 'days'), ('weeks', 'weeks')], default='hours', max_length=12, verbose_name='interval unit')),
                ('repeat', models.PositiveIntegerField(blank=True, null=True, verbose_name='repeat')),
            ],
            options={
                'ordering': ('name',),
                'verbose_name': 'Repeatable Job',
                'verbose_name_plural': 'Repeatable Jobs',
            },
        ),
        migrations.CreateModel(
            name='ScheduledJob',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('name', models.CharField(max_length=128, unique=True, verbose_name='name')),
                ('callable', models.CharField(max_length=2048, verbose_name='callable')),
                ('enabled', models.BooleanField(default=True, verbose_name='enabled')),
                ('queue', models.CharField(choices=[(b'default', b'default'), (b'high', b'high'), (b'low', b'low')], max_length=16, verbose_name='queue')),
                ('job_id', models.CharField(blank=True, editable=False, max_length=128, null=True, verbose_name='job id')),
                ('scheduled_time', models.DateTimeField(verbose_name='scheduled time')),
            ],
            options={
                'ordering': ('name',),
                'verbose_name': 'Scheduled Job',
                'verbose_name_plural': 'Scheduled Jobs',
            },
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-04-30 03:10
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('scheduler', '0002_add_timeout'),
    ]

    operations = [
        migrations.AlterField(
            model_name='repeatablejob',
            name='queue',
            field=models.CharField(max_length=16, verbose_name='queue'),
        ),
        migrations.AlterField(
            model_name='scheduledjob',
            name='queue',
            field=models.CharField(max_length=16, verbose_name='queue'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from datetime import datetime, timedelta

from django.conf import settings
from django.core.exceptions import ValidationError
from django.test import TestCase

import factory
import pytz
from django_rq import job
from scheduler.models import CronJob, RepeatableJob, ScheduledJob


class ScheduledJobFactory(factory.Factory):

    name = factory.Sequence(lambda n: 'Addition {}'.format(n))
    job_id = None
    queue = 'default'
    callable = 'scheduler.tests.test_job'
    enabled = True

    @factory.lazy_attribute
    def scheduled_time(self):
        return datetime.now() + timedelta(days=1)

    class Meta:
        model = ScheduledJob


class RepeatableJobFactory(factory.Factory):

    name = factory.Sequence(lambda n: 'Addition {}'.format(n))
    job_id = None
    queue = 'default'
    callable = 'scheduler.tests.test_job'
    enabled = True
    interval = 1
    interval_unit = 'hours'
    repeat = None

    @factory.lazy_attribute
    def scheduled_time(self):
        return datetime.now() + timedelta(minutes=1)

    class Meta:
        model = RepeatableJob


class CronJobFactory(factory.Factory):

    name = factory.Sequence(lambda n: 'Addition {}'.format(n))
    job_id = None
    queue = 'default'
    callable = 'scheduler.tests.test_job'
    enabled = True
    cron_string = "0 0 * * *"
    repeat = None

    class Meta:
        model = CronJob


@job
def test_job():
    return 1 + 1


test_non_callable = 'I am a teapot'


class TestScheduledJob(TestCase):

    JobClass = ScheduledJob
    JobClassFactory = ScheduledJobFactory

    def test_callable_func(self):
        job = self.JobClass()
        job.callable = 'scheduler.tests.test_job'
        func = job.callable_func()
        self.assertEqual(test_job, func)

    def test_callable_func_not_callable(self):
        job = self.JobClass()
        job.callable = 'scheduler.tests.test_non_callable'
        with self.assertRaises(TypeError):
            job.callable_func()

    def test_clean_callable(self):
        job = self.JobClass()
        job.callable = 'scheduler.tests.test_job'
        assert job.clean_callable() is None

    def test_clean_callable_invalid(self):
        job = self.JobClass()
        job.callable = 'scheduler.tests.test_non_callable'
        with self.assertRaises(ValidationError):
            job.clean_callable()

    def test_clean(self):
        job = self.JobClass()
        job.queue = settings.RQ_QUEUES.keys()[0]
        job.callable = 'scheduler.tests.test_job'
        assert job.clean() is None

    def test_clean_invalid(self):
        job = self.JobClass()
        job.queue = settings.RQ_QUEUES.keys()[0]
        job.callable = 'scheduler.tests.test_non_callable'
        with self.assertRaises(ValidationError):
            job.clean()

    def test_clean_queue_invalid(self):
        job = self.JobClass()
        job.queue = 'xxxxxx'
        job.callable = 'scheduler.tests.test_job'
        with self.assertRaises(ValidationError):
            job.clean()

    def test_is_schedulable_already_scheduled(self):
        job = self.JobClass()
        job.job_id = 'something'
        self.assertFalse(job.is_schedulable())

    def test_is_schedulable_disabled(self):
        job = self.JobClass()
        job.id = 1
        job.enabled = False
        self.assertFalse(job.is_schedulable())

    def test_is_schedulable_enabled(self):
        job = self.JobClass()
        job.id = 1
        job.enabled = True
        self.assertTrue(job.is_schedulable())

    def test_schedule(self):
        job = self.JobClassFactory()
        job.id = 1
        successful = job.schedule()
        self.assertTrue(successful)
        self.assertIsNotNone(job.job_id)

    def test_unschedulable(self):
        job = self.JobClassFactory()
        job.enabled = False
        successful = job.schedule()
        self.assertFalse(successful)
        self.assertIsNone(job.job_id)

    def test_unschedule(self):
        job = self.JobClassFactory()
        job.job_id = 'something'
        successful = job.unschedule()
        self.assertTrue(successful)
        self.assertIsNone(job.job_id)

    def test_unschedule_not_scheduled(self):
        job = self.JobClassFactory()
        successful = job.unschedule()
        self.assertTrue(successful)
        self.assertIsNone(job.job_id)

    def test_schedule_time_utc(self):
        job = self.JobClass()
        est = pytz.timezone('US/Eastern')
        scheduled_time = datetime(2016, 12, 25, 8, 0, 0, tzinfo=est)
        job.scheduled_time = scheduled_time
        utc = pytz.timezone('UTC')
        expected = scheduled_time.astimezone(utc).isoformat()
        self.assertEqual(expected, job.schedule_time_utc().isoformat())

    def test_save_enabled(self):
        job = self.JobClassFactory()
        job.save()
        self.assertIsNotNone(job.job_id)

    def test_save_disabled(self):
        job = self.JobClassFactory()
        job.save()
        job.enabled = False
        job.save()
        self.assertIsNone(job.job_id)

    def test_save_and_schedule(self):
        job = self.JobClassFactory()
        job.id = 1
        job.save()
        is_scheduled = job.is_scheduled()
        self.assertIsNotNone(job.job_id)
        self.assertTrue(is_scheduled)

    def test_delete_and_unschedule(self):
        job_id = 1
        job = self.JobClassFactory()
        job.id = job_id
        job.save()
        is_scheduled = job.is_scheduled()
        self.assertIsNotNone(job.job_id)
        self.assertTrue(is_scheduled)
        scheduler = job.scheduler()
        job.delete()
        is_scheduled = job_id in scheduler
        self.assertFalse(is_scheduled)


class TestRepeatableJob(TestScheduledJob):

    JobClass = RepeatableJob
    JobClassFactory = RepeatableJobFactory

    def test_interval_seconds_weeks(self):
        job = RepeatableJob()
        job.interval = 2
        job.interval_unit = 'weeks'
        self.assertEqual(1209600.0, job.interval_seconds())

    def test_interval_seconds_days(self):
        job = RepeatableJob()
        job.interval = 2
        job.interval_unit = 'days'
        self.assertEqual(172800.0, job.interval_seconds())

    def test_interval_seconds_hours(self):
        job = RepeatableJob()
        job.interval = 2
        job.interval_unit = 'hours'
        self.assertEqual(7200.0, job.interval_seconds())

    def test_interval_seconds_minutes(self):
        job = RepeatableJob()
        job.interval = 15
        job.interval_unit = 'minutes'
        self.assertEqual(900.0, job.interval_seconds())

    def test_repeatable_schedule(self):
        job = self.JobClassFactory()
        job.id = 1
        successful = job.schedule()
        self.assertTrue(successful)
        self.assertIsNotNone(job.job_id)

    def test_repeatable_unschedulable(self):
        job = self.JobClassFactory()
        job.enabled = False
        successful = job.schedule()
        self.assertFalse(successful)
        self.assertIsNone(job.job_id)


class TestCronJob(TestScheduledJob):

    JobClass = CronJob
    JobClassFactory = CronJobFactory

    def test_clean(self):
        job = self.JobClass()
        job.cron_string = '* * * * *'
        job.queue = settings.RQ_QUEUES.keys()[0]
        job.callable = 'scheduler.tests.test_job'
        assert job.clean() is None

    def test_clean_cron_string_invalid(self):
        job = self.JobClass()
        job.cron_string = 'not-a-cron-string'
        job.queue = settings.RQ_QUEUES.keys()[0]
        job.callable = 'scheduler.tests.test_job'
        with self.assertRaises(ValidationError):
            job.clean_cron_string()

    def test_cron_schedule(self):
        job = self.JobClassFactory()
        job.id = 1
        successful = job.schedule()
        self.assertTrue(successful)
        self.assertIsNotNone(job.job_id)

    def test_cron_unschedulable(self):
        job = self.JobClassFactory()
        job.enabled = False
        successful = job.schedule()
        self.assertFalse(successful)
        self.assertIsNone(job.job_id)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-14 12:45
from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    dependencies = [
        ('scheduler', '0003_remove_queue_choices'),
    ]

    operations = [
        migrations.CreateModel(
            name='CronJob',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('name', models.CharField(max_length=128, unique=True, verbose_name='name')),
                ('callable', models.CharField(max_length=2048, verbose_name='callable')),
                ('enabled', models.BooleanField(default=True, verbose_name='enabled')),
                ('queue', models.CharField(max_length=16, verbose_name='queue')),
                ('job_id', models.CharField(blank=True, editable=False, max_length=128, null=True, verbose_name='job id')),
                ('timeout', models.IntegerField(blank=True, help_text="Timeout specifies the maximum runtime, in seconds, for the job before it'll be considered 'lost'. Blank uses the default timeout.", null=True, verbose_name='timeout')),
                ('repeat', models.PositiveIntegerField(blank=True, null=True, verbose_name='repeat')),
                ('cron_string', models.CharField(help_text='Define the schedule in a crontab like syntax.', max_length=64, verbose_name='cron string')),
            ],
            options={
                'ordering': ('name',),
                'verbose_name': 'Cron Job',
                'verbose_name_plural': 'Cron Jobs',
            },
        ),
    ]



from __future__ import unicode_literals
import importlib
from datetime import timedelta

import croniter

from django.conf import settings
from django.core.exceptions import ValidationError
from django.db import models
from django.templatetags.tz import utc
from django.utils.translation import ugettext_lazy as _

import django_rq
from model_utils import Choices
from model_utils.models import TimeStampedModel


class BaseJob(TimeStampedModel):

    name = models.CharField(_('name'), max_length=128, unique=True)
    callable = models.CharField(_('callable'), max_length=2048)
    enabled = models.BooleanField(_('enabled'), default=True)
    queue = models.CharField(_('queue'), max_length=16)
    job_id = models.CharField(
        _('job id'), max_length=128, editable=False, blank=True, null=True)
    timeout = models.IntegerField(
        _('timeout'), blank=True, null=True,
        help_text=_(
            'Timeout specifies the maximum runtime, in seconds, for the job '
            'before it\'ll be considered \'lost\'. Blank uses the default '
            'timeout.'
        )
    )

    def __str__(self):
        return self.name

    def callable_func(self):
        path = self.callable.split('.')
        module = importlib.import_module('.'.join(path[:-1]))
        func = getattr(module, path[-1])
        if callable(func) is False:
            raise TypeError("'{}' is not callable".format(self.callable))
        return func

    def clean(self):
        self.clean_callable()
        self.clean_queue()

    def clean_callable(self):
        try:
            self.callable_func()
        except:
            raise ValidationError({
                'callable': ValidationError(
                    _('Invalid callable, must be importable'), code='invalid')
            })

    def clean_queue(self):
        queue_keys = settings.RQ_QUEUES.keys()
        if self.queue not in queue_keys:
            raise ValidationError({
                'queue': ValidationError(
                    _('Invalid queue, must be one of: {}'.format(
                        ', '.join(queue_keys))), code='invalid')
            })

    def is_scheduled(self):
        return self.job_id in self.scheduler()
    is_scheduled.short_description = _('is scheduled?')
    is_scheduled.boolean = True

    def save(self, **kwargs):
        self.unschedule()
        if self.enabled:
            self.schedule()
        super(BaseJob, self).save(**kwargs)

    def delete(self, **kwargs):
        self.unschedule()
        super(BaseJob, self).delete(**kwargs)

    def scheduler(self):
        return django_rq.get_scheduler(self.queue)

    def is_schedulable(self):
        if self.job_id:
            return False
        return self.enabled

    def schedule(self):
        if self.is_schedulable() is False:
            return False
        kwargs = {}
        if self.timeout:
            kwargs['timeout'] = self.timeout
        job = self.scheduler().enqueue_at(
            self.schedule_time_utc(), self.callable_func(),
            **kwargs
        )
        self.job_id = job.id
        return True

    def unschedule(self):
        if self.is_scheduled():
            self.scheduler().cancel(self.job_id)
        self.job_id = None
        return True

    def schedule_time_utc(self):
        return utc(self.scheduled_time)

    class Meta:
        abstract = True


class ScheduledTimeMixin(models.Model):

    scheduled_time = models.DateTimeField(_('scheduled time'))

    def schedule_time_utc(self):
        return utc(self.scheduled_time)

    class Meta:
        abstract = True


class ScheduledJob(ScheduledTimeMixin, BaseJob):

    class Meta:
        verbose_name = _('Scheduled Job')
        verbose_name_plural = _('Scheduled Jobs')
        ordering = ('name', )


class RepeatableJob(ScheduledTimeMixin, BaseJob):

    UNITS = Choices(
        ('minutes', _('minutes')),
        ('hours', _('hours')),
        ('days', _('days')),
        ('weeks', _('weeks')),
    )

    interval = models.PositiveIntegerField(_('interval'))
    interval_unit = models.CharField(
        _('interval unit'), max_length=12, choices=UNITS, default=UNITS.hours
    )
    repeat = models.PositiveIntegerField(_('repeat'), blank=True, null=True)

    def interval_display(self):
        return '{} {}'.format(self.interval, self.get_interval_unit_display())

    def interval_seconds(self):
        kwargs = {
            self.interval_unit: self.interval,
        }
        return timedelta(**kwargs).total_seconds()

    def schedule(self):
        if self.is_schedulable() is False:
            return False
        kwargs = {
            'scheduled_time': self.schedule_time_utc(),
            'func': self.callable_func(),
            'interval': self.interval_seconds(),
            'repeat': self.repeat
        }
        if self.timeout:
            kwargs['timeout'] = self.timeout
        job = self.scheduler().schedule(**kwargs)
        self.job_id = job.id
        return True

    class Meta:
        verbose_name = _('Repeatable Job')
        verbose_name_plural = _('Repeatable Jobs')
        ordering = ('name', )


class CronJob(BaseJob):

    cron_string = models.CharField(
        _('cron string'), max_length=64,
        help_text=_('Define the schedule in a crontab like syntax.')
    )
    repeat = models.PositiveIntegerField(_('repeat'), blank=True, null=True)

    def clean(self):
        super(CronJob, self).clean()
        self.clean_cron_string()

    def clean_cron_string(self):
        try:
            croniter.croniter(self.cron_string)
        except ValueError as e:
            raise ValidationError({
                'cron_string': ValidationError(
                    _(str(e)), code='invalid')
            })

    def schedule(self):
        if self.is_schedulable() is False:
            return False
        kwargs = {
            'func': self.callable_func(),
            'cron_string': self.cron_string,
            'repeat': self.repeat
        }
        if self.timeout:
            kwargs['timeout'] = self.timeout
        job = self.scheduler().cron(**kwargs)
        self.job_id = job.id
        return True

    class Meta:
        verbose_name = _('Cron Job')
        verbose_name_plural = _('Cron Jobs')
        ordering = ('name', )



from django.forms import ModelForm, ValidationError
from django.utils.timezone import now


class JobAdminForm(ModelForm):

    def clean_scheduled_time(self):
        data = self.cleaned_data['scheduled_time']
        if data < now():
            raise ValidationError("Scheduled time has to be in the future")
        return data



#!/usr/bin/env python

import os
from distutils.core import setup
from setuptools import find_packages


def long_desc(root_path):
    FILES = ['README.md']
    for filename in FILES:
        filepath = os.path.realpath(os.path.join(root_path, filename))
        if os.path.isfile(filepath):
            with open(filepath, mode='r') as f:
                yield f.read()


HERE = os.path.abspath(os.path.dirname(__file__))
long_description = "\n\n".join(long_desc(HERE))


def get_version(root_path):
    with open(os.path.join(root_path, 'scheduler', '__init__.py')) as f:
        for line in f:
            if line.startswith('__version__ ='):
                return line.split('=')[1].strip().strip('"\'')


tests_require = [
    'factory_boy>=2.6.1',
    'psycopg2>=2.6.1',
]

setup(
    name='readwise-django-rq-scheduler',
    version=get_version(HERE),
    description='A database backed job scheduler for Django RQ',
    long_description=long_description,
    packages=find_packages(),
    include_package_data=True,
    author='Chad Shryock',
    author_email='chad@isl.co',
    url='https://github.com/TristanH/django-rq-scheduler',
    zip_safe=True,
    install_requires=[
        'django>=2',
        'django-model-utils>=2.4.0',
        'django-rq>=0.9.0',
        'rq_scheduler>=0.6.0',
        'pytz>=2015.7',
    ],
    tests_require=tests_require,
    extras_require={
        'test': tests_require,
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        'Framework :: Django',
        "Framework :: Django :: 2.0",
        "Framework :: Django :: 3.0",
    ],
)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-04-30 01:15
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('scheduler', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='repeatablejob',
            name='timeout',
            field=models.IntegerField(blank=True, help_text="Timeout specifies the maximum runtime, in seconds, for the job before it'll be considered 'lost'. Blank uses the default timeout.", null=True, verbose_name='timeout'),  # noqa
        ),
        migrations.AddField(
            model_name='scheduledjob',
            name='timeout',
            field=models.IntegerField(blank=True, help_text="Timeout specifies the maximum runtime, in seconds, for the job before it'll be considered 'lost'. Blank uses the default timeout.", null=True, verbose_name='timeout'),  # noqa
        ),
    ]



from __future__ import unicode_literals

from django.conf import settings
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from scheduler.models import CronJob, RepeatableJob, ScheduledJob
from scheduler.forms import JobAdminForm


QUEUES = [(key, key) for key in settings.RQ_QUEUES.keys()]


class QueueMixin(object):
    form = JobAdminForm
    actions = ['delete_model']

    def get_actions(self, request):
        actions = super(QueueMixin, self).get_actions(request)
        del actions['delete_selected']
        return actions

    def get_form(self, request, obj=None, **kwargs):
        queue_field = self.model._meta.get_field('queue')
        queue_field.choices = QUEUES
        return super(QueueMixin, self).get_form(request, obj, **kwargs)

    def delete_model(self, request, obj):
        if hasattr(obj, 'all'):
            for o in obj.all():
                o.delete()
        else:
            obj.delete()
    delete_model.short_description = _("Delete selected %(verbose_name_plural)s")


@admin.register(ScheduledJob)
class ScheduledJobAdmin(QueueMixin, admin.ModelAdmin):
    list_display = (
        'name', 'job_id', 'is_scheduled', 'scheduled_time', 'enabled')
    list_filter = ('enabled', )
    list_editable = ('enabled', )

    readonly_fields = ('job_id', )
    fieldsets = (
        (None, {
            'fields': ('name', 'callable', 'enabled', ),
        }),
        (_('RQ Settings'), {
            'fields': ('queue', 'job_id', ),
        }),
        (_('Scheduling'), {
            'fields': (
                'scheduled_time',
                'timeout',
            ),
            'description': _('Please be aware: Scheduled Time has to be in the future.'),
        }),
    )


@admin.register(RepeatableJob)
class RepeatableJobAdmin(QueueMixin, admin.ModelAdmin):
    list_display = (
        'name', 'job_id', 'is_scheduled', 'scheduled_time', 'interval_display',
        'enabled')
    list_filter = ('enabled', )
    list_editable = ('enabled', )

    readonly_fields = ('job_id', )
    fieldsets = (
        (None, {
            'fields': ('name', 'callable', 'enabled', ),
        }),
        (_('RQ Settings'), {
            'fields': ('queue', 'job_id', ),
        }),
        (_('Scheduling'), {
            'fields': (
                'scheduled_time',
                ('interval', 'interval_unit', ),
                'repeat',
                'timeout',
            ),
            'description': _('Please be aware: Scheduled Time has to be in the future.'),
        }),
    )


@admin.register(CronJob)
class CronJobAdmin(QueueMixin, admin.ModelAdmin):
    list_display = (
        'name', 'job_id', 'is_scheduled', 'cron_string', 'enabled')
    list_filter = ('enabled', )
    list_editable = ('enabled', )

    readonly_fields = ('job_id', )
    fieldsets = (
        (None, {
            'fields': ('name', 'callable', 'enabled', ),
        }),
        (_('RQ Settings'), {
            'fields': ('queue', 'job_id', ),
        }),
        (_('Scheduling'), {
            'fields': (
                'cron_string',
                'repeat',
                'timeout',
            ),
        }),
    )
